#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
###
# Application: wah!cade
# File:        filters.py
# Description: routines to read / write mamewah filters and lists
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import os
import sys
import glob
import codecs
import ConfigParser
#thanks to Trent Mick (http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/475126)
try:
    import xml.etree.cElementTree as ET # python >=2.5 C module
except ImportError:
    try:
        import xml.etree.ElementTree as ET # python >=2.5 pure Python module
    except ImportError:
        try:
            import cElementTree as ET # effbot's C module
        except ImportError:
            try:
                import elementtree.ElementTree as ET # effbot's pure Python module
            except ImportError:
                try:
                    import lxml.etree as ET # ElementTree API using libxml2
                except ImportError:
                    import warnings
                    warnings.warn("could not import ElementTree "
                                  "(http://effbot.org/zone/element-index.htm)")
from constants import *
_ = gettext.gettext

#mameinfo file
_mameinfo_file = 'mameinfo.xml'
#category
_catver_ini = 'catver.ini'
#filter sections
_sections = [
    'filter_type',
    'year',
    'manufacturer',
    'driver',
    'display_type',
    'screen_type',
    'controller_type',
    'driver_status',
    'colour_status',
    'sound_status',
    'category']
#controller types
_controllers = {
    'joy2way': u'2-Way Joystick',
    'vjoy2way': u'Vertical 2-Way Joystick',
    'joy4way': u'4-Way Joystick',
    'joy8way': u'8-Way Joystick',
    'stick': u'Analogue Joystick (or ~270\xb0 Wheel)',
    'doublejoy2way': u'Double 2-Way Joysticks',
    'vdoublejoy2way': u'Double Vertical 2-Way Joysticks',
    'doublejoy4way': u'Double 4-Way Joysticks',
    'doublejoy8way': u'Double 8-Way Joysticks',
    'lightgun': u'Light-Gun',
    '': u'None',
    'paddle': u'Paddle (or ~270\xb0 Wheel)',
    'dial': u'Spinner (or 360\xb0 Wheel)',
    'trackball': u'Trackball'
}


def get_dat_game_item(dat_filename):
    """read dat file"""
    #get category info
    catver = get_catver_ini(_catver_ini)
    #needs to be an iterator returning next item in file each time
    #
    f = open(dat_filename, 'r')
    for line in f:
        line = line.strip()
        #line = unicode(line, "cp1252").encode("utf-8")
        #look for start of item
        if line == 'game (':
            #setup empty
            d = {'rom_name': '',
                'game_name': '',
                'year': '',
                'manufacturer': '',
                'clone_of': '',
                'rom_of': '',
                'display_type': '',
                'screen_type': '',
                'controller_type': '',
                'driver_status': '',
                'colour_status': '',
                'sound_status': '',
                'category': ''}
            #get item attribs
            while line != ')':
                #
                if line.startswith('\t'):
                    key, value = line.split(' ', 1)
                    value = value.strip('"')
                    #print "key=", key
                    if key == '\tname':
                        d['rom_name'] = value
                    elif key == '\tcloneof':
                        d['clone_of'] = value
                    elif key == '\t':
                        d['rom_of'] = value
                    elif key == '\tdescription':
                        d['game_name'] = value
                    elif key == '\tyear':
                        d['year'] = value
                    elif key == '\tmanufacturer':
                        d['manufacturer'] = value
                    elif key == '\tvideo':
                        vid = {}
                        l = value[2:-2].split()
                        for i in range(0, len(l) - 1, 2):
                            vid[l[i]] = l[i+1]
                        if vid.has_key('screen'):
                            d['display_type'] = vid['screen']
                        if vid.has_key('orientation'):
                            d['screen_type'] = vid['orientation']
                    elif key == '\tdriver':
                        driver = {}
                        l = value[2:-2].split()
                        for i in range(0, len(l) - 1, 2):
                            driver[l[i]] = l[i+1]
                        if driver.has_key('status'):
                            d['driver_status'] = driver['status']
                        if driver.has_key('color'):
                            d['colour_status'] = driver['color']
                        if driver.has_key('sound'):
                            d['sound_status'] = driver['sound']
                    elif key == '\tinput':
                        ctrl = {}
                        l = value[2:-2].split()
                        for i in range(0, len(l) - 1, 2):
                            ctrl[l[i]] = l[i+1]
                        if ctrl.has_key('control'):
                            d['controller_type'] = _controllers[ctrl['control']]
                    #get category
                    try:
                        d['category'] = catver.get('Category', d['rom_name'].lower())
                    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
                        d['category'] = ''
                #read next time
                line = f.next()
                line = line.strip('\n')
            #yield back to calling fund
            yield d
    #done
    f.close()

def get_xml_game_item(xml_filename):
    """get game bits"""
    #get category info
    catver = get_catver_ini(_catver_ini)
    for event, game_element in ET.iterparse(xml_filename):
        #setup empty
        d = {'rom_name': '',
            'game_name': '',
            'year': '',
            'manufacturer': '',
            'clone_of': '',
            'rom_of': '',
            'display_type': '',
            'screen_type': '',
            'controller_type': '',
            'driver_status': '',
            'colour_status': '',
            'sound_status': '',
            'category': ''}
        if game_element.tag == 'game':
            #get sub elements
            vid = game_element.find('video')
            ctrl = game_element.find('input')
            driver = game_element.find('driver')
            year = game_element.findtext('year')
            manu = game_element.findtext('manufacturer').title()
            desc = game_element.findtext('description')
            #create dict
            d['rom_name'] = game_element.attrib['name'].upper()
            if desc is not None:
                d['game_name'] = desc
            if year is not None:
                d['year'] = year
            if manu is not None:
                d['manufacturer'] = manu
            if 'cloneof' in game_element.keys():
                d['clone_of'] = game_element.attrib['cloneof'].upper()
            if 'romof' in game_element.keys():
                d['rom_of'] = game_element.attrib['romof'].upper()
            if vid is not None:
                d['display_type'] = vid.attrib['screen'].title()
                d['screen_type'] = vid.attrib['orientation'].title()
            if ctrl is not None:
                if 'control' in ctrl.keys():
                    d['controller_type'] = _controllers[ctrl.attrib['control']]
            if driver is not None:
                d['driver_status'] = 'Status %s' % (driver.attrib['status'].title())
                d['colour_status'] = 'Color %s' % (driver.attrib['color'].title())
                d['sound_status'] = 'Sound %s' % (driver.attrib['sound'].title())
            try:
                d['category'] = catver.get('Category', d['rom_name'].lower())
            except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
                d['category'] = ''
            #done
            yield d
    game_element.clear() # won't need this any more

def get_catver_ini(category_file):
    """load catver.ini file"""
    if not os.path.exists(category_file):
        print _("Error: category info file: [%s] does not exist" % (category_file))
    catver = ConfigParser.SafeConfigParser()
    catver.read(category_file)
    return catver

def create_initial_filter(info_filename, filter_filename, list_filename, emu_ini):
    """create a new mamewah .ftr & .lst file"""
    #create empty dict
    mwfilter = {}
    for s in _sections[1:]:
        mwfilter[s] = []
    #open new list file
    f = codecs.open(list_filename, mode='w', encoding='cp1252')
    if os.path.exists(info_filename):
        #create iterator for game info
        r, ext = os.path.splitext(info_filename)
        if ext == '.xml':
            mi = get_xml_game_item(info_filename)
        else:
            mi = get_dat_game_item(info_filename)
    elif emu_ini.get('list_generation_method') != 'rom_folder':
        #file not found
        print _("Error: emulator info file: [%s] does not exist" % (info_filename))
        print _("       cannot create initial filters!")
        mi = None
        f.close()
        return
    #get rom path & ext, etc
    rom_path = emu_ini.get('rom_path')
    if not os.path.exists(emu_ini.get('rom_path')):
        print _("Error: rom path: [%s] does not exist" % (emu_ini.get('rom_path')))
    rom_ext = emu_ini.get('rom_extension')
    rom_list_gen = 'rom_folder' in emu_ini.get('list_generation_method')
    if emu_ini.get('list_generation_method') == 'rom_folder':
        #just rom folder, so create a basic lst file
        #print "rom path=", os.path.join(rom_path, '*.%s' % (rom_ext))
        roms = glob.glob(os.path.join(rom_path, '*.%s' % (rom_ext)))
        roms.sort()
        #print "#roms=",len(roms)
        for rom_filename in roms:
            rom_name = os.path.splitext(os.path.basename(rom_filename))[0]
            gd = {'rom_name': rom_name,
                'game_name': rom_name,
                'year': '',
                'manufacturer': '',
                'clone_of': '',
                'rom_of': '',
                'display_type': '',
                'screen_type': '',
                'controller_type': '',
                'driver_status': '',
                'colour_status': '',
                'sound_status': '',
                'category': ''}
            add_game_to_filtered_list(gd, f)
        #done
        f.close()
        return
    #use dat / xml info iterator
    while True:
        try:
            gd = mi.next()
            #write gd to list file
            try:
                #check rom actually exists in rom folder location?
                if rom_list_gen:
                    rom_filename = os.path.join(rom_path, gd['rom_name'])
                    if rom_ext != '':
                        rom_filename = '%s.%s' % (rom_filename, rom_ext)
                    if os.path.exists(rom_filename) or os.path.exists(rom_filename.lower()):
                        #it does, add game to list
                        add_game_to_filtered_list(gd, f)
                    #else:
                    #    print "%s  does not exist" % (rom_filename)
                else:
                    #add game to list
                    add_game_to_filtered_list(gd, f)
            except TypeError, msg:
                print msg
                print gd
                sys.exit(1)
        except StopIteration:
            break
        #for each key in sections
        for key in _sections[1:]:
            if gd.has_key(key):
                if gd[key] not in mwfilter[key]:
                    mwfilter[key].append(gd[key])
    #done list file
    f.close()
    #write filter file
    f = codecs.open(filter_filename, mode='w', encoding='cp1252')
    f.write('1\n\n')
    for key in _sections[1:]:
        l = mwfilter[key]
        l.sort()
        for item in l:
            if item is not None and item != '':
                try:
                    f.write('%s\nTrue\n' % (item))
                except UnicodeEncodeError, msg:
                    print "unicode encode error"
                    print "  msg=", msg
                    #print "  error item=", item
        f.write('\n')
    f.close()

def read_filter(filter_file):
    """read a mamewah .ftr file"""
    #create a filter first if it doesn't already exist
    #if not os.path.isfile(filter_file):
    #    create_filter('xml', _mameinfo_file, filter_file)
    #init
    gap_num = 0
    d = {}
    l=[]
    #open file
    try:
        f = open(filter_file, 'r')
    except IOError:
        print _('Error: filter file: [%s] does not exist' % (filter_file))
        return {}
    #get type
    line = f.next()
    line = line.strip()
    d[_sections[0]] = line
    #print "filter type=", d
    f.next()
    #for each line in file
    for line in f:
        line = line.strip()
        if line == '':
            #end of section, add current list (as a dictionary) to filter
            gap_num += 1
            d[_sections[gap_num]] = dict(l)
            l = []
        else:
            #read name & true / false setting
            try:
                next_line = f.next().strip()
            except StopIterator:
                next_line = 'False'
            filt_item = [line, next_line.lower() == 'true']
            l.append(filt_item)
    #done, close file & return dictionary
    f.close()
    return d

def write_filter(filter_spec, filter_filename):
    """write a filter file from a given spec"""
    #f = codecs.open(filter_filename, mode='w', encoding='cp1252')
    f = open(filter_filename, 'w')
    #write filter type
    f.write('%s\n\n' % filter_spec[_sections[0]])
    #write other sections
    for key in _sections[1:]:
        l = filter_spec[key].items()
        l.sort()
        for item, value in l: #filter_spec[key].items():
            f.write('%s\n%s\n' % (item, str(value)))
        f.write('\n')
    #done
    f.close()

def create_filtered_list(original_list_filename, filter_spec, list_filename):
    """create a mamewah .lst file from emu-0.lst and the given spec"""
    #get category info
    catver = get_catver_ini(_catver_ini)
    #open empty list file
    f = codecs.open(list_filename, mode='w', encoding='cp1252')
    #get all games
    all_games = read_filtered_list(original_list_filename)
    all_games_roms = [l[1] for l in all_games]
    #for each item in iterator
    for game_item in all_games:
        gd = get_game_dict(game_item)
        #match filter conditions
        matched = True
        #check filter type section
        if filter_spec['filter_type'] != FTR_CLONES_YES and gd['clone_of'] != '':
            #print "gd clone=", gd['clone_of']
            matched = False
        if (filter_spec['filter_type'] == FTR_CLONES_BETTER and
              gd['clone_of'] != '' and
              'good' in gd['driver_status'].lower()):
            #see if original's driver status is worse than this one
            try:
                orig_rom = get_game_dict(all_games[all_games_roms.index(gd['clone_of'])])
            except ValueError:
                print "rom=%s: can't find clone of: %s" % (gd['rom_name'], gd['clone_of'])
                matched = True
            if 'good' not in orig_rom['driver_status'].lower():
                matched = True
        #check the other sections
        if matched:
            #for each section
            for section in _sections[1:]:
                if filter_spec.has_key(section) and gd.has_key(section):
                    #if spec and current game have same section
                    if filter_spec[section].has_key(gd[section]):
                        if not filter_spec[section][gd[section]]:
                            #don't match if spec entry for game not set (i.e. False)
                            matched = False
                    if gd[section] is None or gd[section] == '':
                        matched = False
        if matched:
            #write output
            add_game_to_filtered_list(gd, f)
    #done, close file
    f.close()

def read_filtered_list(list_filename):
    """read a mamewah .lst file"""
    l = []
    lines = open(list_filename, 'r').readlines()
    lines = [s.strip() for s in lines]
    #convert string to utf-8 (stops pygtk unicode errors)
    lines = [unicode(s, "cp1252").encode("utf-8") for s in lines]
    #for each game...
    for i in range(0, len(lines), 13):
        item = []
        for j in range(13):
            item.append(lines[i + j])
        #reverse first two columns
        item2 = [item[1], item[0]]
        item2.extend(item[2:])
        l.append(item2)
    #done
    return l

def add_game_to_filtered_list(gd, file_obj=None, list_filename=None):
    """add game info in given dictionary to specified file or .lst file"""
    if list_filename is not None:
        #open file for appending if necessary
        file_obj = codecs.open(list_filename, mode='a', encoding='cp1252')
    #write dictionary to file
    file_obj.writelines([
        '%s\n' % gd['rom_name'],
        '%s\n' % gd['game_name'],
        '%s\n' % gd['year'],
        '%s\n' % gd['manufacturer'],
        '%s\n' % gd['clone_of'],
        '%s\n' % gd['rom_of'],
        '%s\n' % gd['display_type'],
        '%s\n' % gd['screen_type'],
        '%s\n' % gd['controller_type'],
        '%s\n' % gd['driver_status'],
        '%s\n' % gd['colour_status'],
        '%s\n' % gd['sound_status'],
        '%s\n' % gd['category']
    ])
    #done
    if list_filename is not None:
        file_obj.close()

def write_filtered_list(list_filename , list_items):
    """write the given list items into the specified .lst file"""
    f = open(list_filename, 'w')
    for game_info in list_items:
        f.writelines([
            '%s\n' % game_info[GL_ROM_NAME],
            '%s\n' % game_info[GL_GAME_NAME],
            '%s\n' % game_info[GL_YEAR],
            '%s\n' % game_info[GL_MANUFACTURER],
            '%s\n' % game_info[GL_CLONE_OF],
            '%s\n' % game_info[GL_ROM_OF],
            '%s\n' % game_info[GL_DISPLAY_TYPE],
            '%s\n' % game_info[GL_SCREEN_TYPE],
            '%s\n' % game_info[GL_CONTROLLER_TYPE],
            '%s\n' % game_info[GL_DRIVER_STATUS],
            '%s\n' % game_info[GL_COLOUR_STATUS],
            '%s\n' % game_info[GL_SOUND_STATUS],
            '%s\n' % game_info[GL_CATEGORY]
        ])
    #done
    f.close()

def read_fav_list(favlist_filename):
    """reads a .fav list"""
    d = {}
    lines = open(favlist_filename, 'r').readlines()
    lines = [s.strip() for s in lines]
    #convert string to utf-8 (stops pygtk unicode errors)
    lines = [unicode(s, "cp1252").encode("utf-8") for s in lines]
    #for each game...
    for i in range(0, len(lines), 4):
        item = []
        for j in range(4):
            item.append(lines[i + j])
        #add to favs dictionary
        d[item[FAV_ROM_NAME]] = [
            item[FAV_ROM_NAME],
            item[FAV_GAME_NAME],
            int(item[FAV_TIMES_PLAYED]),
            int(item[FAV_MINS_PLAYED])]
    #done
    return d

def write_fav_list(favlist_filename, favs):
    """writes a .fav list"""
    f = open(favlist_filename, 'w')
    for fav_key in favs.keys():
        f.writelines([
            '%s\n' % fav_key,
            '%s\n' % favs[fav_key][FAV_GAME_NAME],
            '%s\n' % favs[fav_key][FAV_TIMES_PLAYED],
            '%s\n' % favs[fav_key][FAV_MINS_PLAYED]
        ])
    f.close()

def get_game_dict(game_item):
    """return dictionary of info for given game item"""
    gd = {
        'rom_name': game_item[GL_ROM_NAME],
        'game_name': game_item[GL_GAME_NAME],
        'year': game_item[GL_YEAR],
        'manufacturer': game_item[GL_MANUFACTURER],
        'clone_of': game_item[GL_CLONE_OF],
        'rom_of': game_item[GL_ROM_OF],
        'display_type': game_item[GL_DISPLAY_TYPE],
        'screen_type': game_item[GL_SCREEN_TYPE],
        'controller_type': game_item[GL_CONTROLLER_TYPE],
        'driver_status': game_item[GL_DRIVER_STATUS],
        'colour_status': game_item[GL_COLOUR_STATUS],
        'sound_status': game_item[GL_SOUND_STATUS],
        'category': game_item[GL_CATEGORY]
    }
    return gd


if __name__ == '__main__':
    import time
    from mamewah_ini import MameWahIni
    #set vars
    CONFIG_DIR = os.path.join(sys.path[0], 'config')
    cdir = sys.path[0]
    _mameinfo_file = '' #os.path.join(cdir, 'mameinfo.xml')
    _catver_ini = '' #os.path.join(cdir, 'catver.ini')

    #set to unicode encoding
    try:
        sys.setappdefaultencoding('utf-8')
    except AttributeError:
        pass

    #create filter from xml
    #t1 = time.time()
    emu_ini = MameWahIni(os.path.join(CONFIG_DIR, 'ini', 'zsnes.ini'))
    create_initial_filter(
        _mameinfo_file,
        os.path.join(CONFIG_DIR, 'files', 'zsnes-0.ftr'),
        os.path.join(CONFIG_DIR, 'files', 'zsnes-0.lst'),
        emu_ini)
    #t2 = time.time()

    #create filter from dat
    #create_initial_filter(
    #    _mamewah_dat,
    #    os.path.join(cdir, 'test-0.dat.ftr'),
    #    os.path.join(cdir, 'test-0.dat.lst'))
    #t3 = time.time()

    #print "create_filter: xml took:", t2-t1
    #print "create_filter: dat took:", t3-t2


    #read filtered list
    #filter_spec  = read_filter(os.path.join(CONFIG_DIR, 'files', 'mame-1.ftr'))
    #filter_spec  = read_filter(os.path.join(CONFIG_DIR, 'files', 'mame-14.ftr'))
    #print "keys=", filter_spec.keys()
    #print "type=", filter_spec['filter_type']#.keys()
    #write_filter(filter_spec, os.path.join(cdir, 'test-2.ftr'))

    #create a filtered list
    #filter_spec = {
    #    'year': [['1980', True], ['1981', False], ['1982', True]],
    #    'manufacturer': [['Capcom (Taito America license)', True], ['Capcom / Sega', False], ['Namco', True]]
    #}
    #t1 = time.time()
    #create_filtered_list(
    #    os.path.join(CONFIG_DIR, 'files', 'mame-0.lst'),
    #    filter_spec,
    #    os.path.join(CONFIG_DIR, 'files', 'mame-1.lst'))
    #t2 = time.time()
    #print "create_filtered_list: took:", t2-t1

    #read a filtered list
    #l = read_filtered_list(os.path.join(CONFIG_DIR, 'files', 'mame-1.lst')
    #print l

    #read fav list
    #fav_filename = os.path.join(CONFIG_DIR, 'files', 'mame.fav')
    #d = read_fav_list(fav_filename)
    #print d['CHQFLAG']

    #update fav list
    #d['CHQFLAG'][FAV_TIMES_PLAYED] += 1
    #print d['CHQFLAG']
    #write_fav_list(fav_filename, d)

    #test dat file reading
    #mi = get_dat_game_item(_mamewah_dat)
    #while 1:
    #    try:
    #        d = mi.next()
    #        print d
    #    except StopIteration:
    #        print "finished"
    #        break

