#!/usr/bin/env python
# -*- coding: UTF-8 -*-
#
###
# Application: wah!cade
# File:        layout_props.py
# Description: Wah!Cade Layout Properties Dialog
# Copyright (c) 2006   Andy Balcombe <http://www.anti-particle.com>
###
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
import os
import sys
import glob
import shutil

from constants import *
from glade2 import *
from mamewah_ini import MameWahIni
from key_consts import mamewah_keys
from wc_common import WahCade
_ = gettext.gettext

_mouse_ctrls = {
    'Mouse Button 1': ['MOUSE_BUTTON0'],
    'Mouse Button 2': ['MOUSE_BUTTON1'],
    'Mouse Button 3': ['MOUSE_BUTTON2'],
    'Mouse Button 4': ['MOUSE_BUTTON3'],
    'Mouse Button 5': ['MOUSE_BUTTON4'],
    'Scroll Wheel Down': ['MOUSE_SCROLLDOWN'],
    'Scroll Wheel Up': ['MOUSE_SCROLLUP']
}

class DlgKeyPress(GladeSupport, WahCade):
    """wahcade setup - key press dialog"""
    def __init__(self, glade_filename, window_name, tvw_path, key_changed_cb):
        """build the dialog"""
        GladeSupport.__init__(self, glade_filename, window_name, APP_NAME)
        #setup events
        self.dlgKeyPress.add_events(
            gtk.gdk.SCROLL_MASK |
            gtk.gdk.BUTTON_PRESS_MASK |
            gtk.gdk.KEY_PRESS)
        self.dlgKeyPress.connect('key-press-event', self.on_dlgKeyPress_key_press)
        self.dlgKeyPress.connect('button-press-event', self.on_dlgKeyPress_key_press)
        self.dlgKeyPress.connect('scroll-event', self.on_dlgKeyPress_key_press)
        self.key_changed_cb = key_changed_cb
        self.tvw_path = tvw_path
        self.rev_mouse_ctrls = self.invert_dictionary_with_lists(_mouse_ctrls)
        #print "self.rev_mouse_ctrls=", self.rev_mouse_ctrls
        
    def on_btnClear_clicked(self, *args):
        """clear clicked"""
        self.dlgKeyPress.destroy()
        self.key_changed_cb(self.tvw_path, [], '') 
        
    def on_btnCancel_clicked(self, *args):
        """cancel clicked"""
        self.dlgKeyPress.destroy()
        
    def on_dlgKeyPress_key_press(self, widget, event, *args):
        """a key has been pressed"""
        mw_keys = []
        if event.type == gtk.gdk.KEY_PRESS:
            #keyboard pressed, get gtk keyname
            keyname = gtk.gdk.keyval_name(event.keyval).lower()
            #get mamewah keyname
            if mamewah_keys.has_key(keyname):
                mw_keys = mamewah_keys[keyname]
        elif event.type == gtk.gdk.BUTTON_PRESS:
            #mouse click
            mw_keys = ['MOUSE_BUTTON%s' % (event.button - 1)]
        elif event.type == gtk.gdk.SCROLL: 
            #mouse scroll wheel
            if event.direction in [gtk.gdk.SCROLL_UP, gtk.gdk.SCROLL_LEFT]:
                mw_keys = ['MOUSE_SCROLLUP']
            elif event.direction in [gtk.gdk.SCROLL_DOWN, gtk.gdk.SCROLL_RIGHT]:
                mw_keys = ['MOUSE_SCROLLDOWN']
        else:
            return
        #get mouse action description
        if event.type in [gtk.gdk.BUTTON_PRESS, gtk.gdk.SCROLL]:
            if self.rev_mouse_ctrls.has_key(mw_keys[0]):
                keyname = self.rev_mouse_ctrls[mw_keys[0]]
        #got something valid?
        if keyname == '' or mw_keys == []:
            #unrecognized / unmapped key / mouse 
            dlg = gtk.MessageDialog(
                self.dlgKeyPress,
                gtk.DIALOG_MODAL,
                gtk.MESSAGE_WARNING,
                gtk.BUTTONS_OK,
                _('Could not recognize key / mouse action'))
            resp = dlg.run()
            dlg.destroy()
            return
        if len(mw_keys) > 0:
            #update list
            self.dlgKeyPress.destroy()
            self.key_changed_cb(self.tvw_path, mw_keys, keyname)
            
        
class WinSetup(GladeSupport, WahCade):
    """wahcade setup - main window"""
    def __init__(self, glade_filename, window_name, config_opts, config_args):
        """build the window"""
        WahCade.__init__(self)
        GladeSupport.__init__(self, glade_filename, window_name, APP_NAME)
        #command-line options
        self.config_opts = config_opts
        #set default config location (create / update as necessary)
        self.config_dir = os.path.expanduser('~/.wahcade')
        self.update_user_config()
        #keys list
        self.tvwKeys, self.lsKeys, self.tvwsKeys = self.setup_treeview(
            columns = ['Function', 'Key'], 
            column_types = [gobject.TYPE_STRING, gobject.TYPE_STRING],
            container = self.scwKeys,
            resizeable_cols = False)
        self.tvwKeys.connect('row-activated', self.on_tvwKeys_row_activated)
        tt = gtk.Tooltips()
        tt.set_tip(self.tvwKeys, _('Double-Click a row to change a Key...'))
        #get ini files
        self.wahcade_ini = MameWahIni(os.path.join(self.config_dir, 'wahcade.ini'))
        self.histview_ini = MameWahIni(os.path.join(self.config_dir, 'histview.ini'))
        self.cpviewer_ini = MameWahIni(os.path.join(self.config_dir, 'cpviewer.ini'))
        self.ctrlr_ini = MameWahIni(os.path.join(self.config_dir, 'ctrlr', 'default.ini'), 'ctrlr')
        #emu stuff
        self.emu_list_gen_types = [
            [['rom_folder'], 'Rom Directory'],
            [['rom_folder_vs_listxml', 'list_xml'], 'XML File'],
            [['rom_folder_vs_dat_file', 'dat_file'], 'DAT File']]
        self.emu_scrsave_types = [
            ['blank_screen', 'Blank Screen'], 
            ['slideshow', 'Slide Show'],
            ['movie', 'Movies']]
        self.emu_list_types = [
            ['normal', 'Normal'],
            ['most_played', 'Most Played'],
            ['longest_played', 'Longest Played']]
        self.emu_artwork_txe = [
            self.txeEmuArt1, self.txeEmuArt2, self.txeEmuArt3, self.txeEmuArt4,
            self.txeEmuArt5, self.txeEmuArt6, self.txeEmuArt7, self.txeEmuArt8,
            self.txeEmuArt9, self.txeEmuArt10]
        self.emu_artwork_btn = [
            self.btnEmuArt1, self.btnEmuArt2, self.btnEmuArt3, self.btnEmuArt4,
            self.btnEmuArt5, self.btnEmuArt6, self.btnEmuArt7, self.btnEmuArt8,
            self.btnEmuArt9, self.btnEmuArt10]
        #setup combo boxes
        self.setup_combo_box(self.cboEmuScrSaver, [r[1] for r in self.emu_scrsave_types])
        self.setup_combo_box(self.cboEmuListGen, [r[1] for r in self.emu_list_gen_types])
        self.setup_combo_box(self.cboEmuListType, [r[1] for r in self.emu_list_types])
        #load settings
        self.load_settings()
        self.setup_altered = False

    def on_winSetup_delete_event(self, *args):
        """done, quit the application"""
        if self.setup_altered:
            dlg = gtk.MessageDialog(
                self.winSetup,
                gtk.DIALOG_MODAL,
                gtk.MESSAGE_QUESTION,
                gtk.BUTTONS_YES_NO,
                _('Save changes?'))
            resp = dlg.run()
            if resp == gtk.RESPONSE_YES:
                self.save_emulator_list_settings()
                self.save_emulator_settings()
                self.save_wahcade_settings()
            dlg.destroy()
        #exit gtk loop
        gtk.main_quit()
        return False
        
    def on_Setup_changed(self, widget, *args):
        """widget has been modified, update altered flag"""
        self.setup_altered = True
        
    def on_mnuFSave_activate(self, *args):
        """save settings"""
        self.save_emulator_list_settings()
        self.save_emulator_settings()
        self.save_wahcade_settings()
        self.setup_altered = False

    def on_mnuFQuit_activate(self, *args):
        """quit"""
        self.on_winSetup_delete_event()
        
    def on_mnuHAbout_activate(self, *args):
        """about dialog"""
        self.show_about_dialog('Wah!Cade Setup', self.config_dir)

    def on_tvwKeys_row_activated(self, widget, path, column):
        """keys list double-clicked"""
        #show key press dialog
        dlg = DlgKeyPress(SETUP_GLADE_FILE, 'dlgKeyPress', path, self.on_tvwKeys_change)
        
    def on_tvwKeys_change(self, path, mw_keys, keyname):
        """update key list with new keyname"""
        #clear
        if mw_keys == [] and keyname == '':
            self.ctrlr_ini.set(self.lsKeys[path][0], '')
            self.lsKeys[path][1] = ''
            return
        #get existing actions for key
        mw_functions = self.ctrlr_ini.reverse_get(mw_keys[0])
        #print "mw_functions=", mw_functions
        if mw_functions != []:
            #re-assign?
            key_actions = ''
            for f in mw_functions:
                key_actions += '\t%s\n' % f
            msg = _('"%s" is already assigned to the following actions:\n%s\n' % (
                keyname.upper(), 
                key_actions))
            msg += _('Keep existing assignments?')
            dlg = gtk.MessageDialog(
                self.winSetup,
                gtk.DIALOG_MODAL,
                gtk.MESSAGE_QUESTION,
                gtk.BUTTONS_YES_NO,
                msg)
            resp = dlg.run()
            dlg.destroy()
            if resp == gtk.RESPONSE_NO:
                #remove from each old assignment
                for mrf in mw_functions:
                    v = self.ctrlr_ini.get(mrf)
                    lv = v.strip(' "').split(' | ')
                    #print "mrf=",mrf, v, lv
                    lv.remove(mw_keys[0])
                    s = ''
                    for v in lv:
                        s = '%s | ' % v
                    if lv != []:
                        s = '"%s"' % s[:-3]
                    self.ctrlr_ini.set(mrf, s)
                    #print "s=",s
        #update keys for selected action
        existing_keys = self.ctrlr_ini.get(self.lsKeys[path][0])
        if existing_keys == '':
            self.ctrlr_ini.set(self.lsKeys[path][0], mw_keys[0])
        else:
            self.ctrlr_ini.set(self.lsKeys[path][0], '"%s | %s"' % (existing_keys.strip('"'), mw_keys[0]))
        self.setup_altered = True
        #redisplay list
        self.populate_keys()
        
    def on_cboEmu_changed(self, *args):
        """emulator combo"""
        #save current emulator settings?
        if self.current_emu:
            self.save_emulator_settings()
            if self.current_emu_list:
                #save emu list settings
                self.save_emulator_list_settings()
                self.current_emu_list = None
        #change to new emulator
        if self.cboEmu.get_active() >= 0:
            self.current_emu = self.emu_lists[self.cboEmu.get_active()]
            self.load_emulator_settings(self.current_emu[1], self.current_emu[2])

    def on_cboEmuLists_changed(self, *args):
        """emulator list combo"""
        if self.current_emu_list:
            #save emu list settings
            self.save_emulator_list_settings()
        #get settings for current emu list
        idx = self.cboEmuLists.get_active()
        self.current_emu_list = self.emu_game_lists[idx]
        if idx >= 0:
            self.load_emulator_list_settings(idx, self.current_emu_list[1]) 
        
    def on_btnWCOpenLayoutDir_clicked(self, widget, *args):
        """open layout dir"""
        self.open_dir_dialog(
            self.get_path(os.path.join(self.config_dir, 'layouts', self.txeWCLayoutDir.get_text())),
            'Set Layout Directory', 
            self.set_layout_dir)
            
    def set_layout_dir(self, new_dir):
        """set layout directory"""
        self.txeWCLayoutDir.set_text(os.path.basename(new_dir))

    def on_btnEmuNew_clicked(self, *args):
        """add new set of emulator files"""
        self.save_layout_dialog(
            os.path.join(self.config_dir, 'ini', 'new_emu.ini'),
            _('Create New Emulator file'),
            self.create_new_emulator_files)
            
    def on_btnEmuListNew_clicked(self, *args):
        """add new emulator list files"""
        next_emu_list_num = len(self.cboEmuLists.get_model())
        emu_ini_name = self.current_emu[1]
        #print "self.current_emu[1]=",self.current_emu[1]
        #copy template to ini dir
        shutil.copy2(
            os.path.join(APP_PATH, 'templates', 'new_emulator-1.ini'), 
            os.path.join(self.config_dir, 'ini', '%s-%s.ini' % (emu_ini_name, next_emu_list_num)))
        #save & then reload
        self.save_emulator_list_settings()
        self.current_emu_list = None
        self.load_emulator_settings(self.current_emu[1], self.current_emu[2], next_emu_list_num)
            
    def on_btnEmuExeOpen_clicked(self, widget, *args):
        """emu executable"""
        self.open_file_dialog(
            self.get_path(self.txeEmuExe.get_text()),
            _('Set Emulator Executable'), 
            self.txeEmuExe.set_text)
        
    def on_btnEmuRomDir_clicked(self, widget, *args):
        """emu rom dir"""
        self.open_dir_dialog(
            self.txeEmuRomDir.get_text(),
            _('Set Rom Directory'), 
            self.txeEmuRomDir.set_text)
            
    def on_btnEmuScrMovieDir_clicked(self, widget, *args):
        """emu screen saver movie dir"""
        self.open_dir_dialog(
            self.txeEmuScrMovieDir.get_text(),
            _('Set Screen Saver Movie Directory'), 
            self.txeEmuScrMovieDir.set_text)
        
    def on_btnEmuArt_clicked(self, widget, *args):
        """set emulator artwork dir"""
        idx = self.emu_artwork_btn.index(widget)
        self.open_dir_dialog(
            self.get_path(self.emu_artwork_txe[idx].get_text()),
            _('Set Artwork #%s Directory') % (idx + 1), 
            self.emu_artwork_txe[idx].set_text)
            
    def on_btnEmuMovieDir_clicked(self, widget, *args):
        """emu movie dir"""
        self.open_dir_dialog(
            self.txeEmuMovieDir.get_text(),
            _('Set Movie Directory'), 
            self.txeEmuMovieDir.set_text)
        
    def on_btnMameXMLFile_clicked(self, widget, *args):
        """set mame xml info file"""
        self.open_file_dialog(
            self.get_path(self.txeMameXMLFile.get_text()),
            _('Set Mame XML / Data File'), 
            self.txeMameXMLFile.set_text)
            
    def on_btnMameCatver_clicked(self, widget, *args):
        """set mame xml catver.ini file"""
        self.open_file_dialog(
            self.get_path(self.txeMameCatver.get_text()),
            _('Set Mame Category Version File'), 
            self.txeMameCatver.set_text)
            
    def on_btnHstDatFile_clicked(self, widget, *args):
        """set history viewer history.dat file"""
        self.open_file_dialog(
            self.get_path(self.txeHstDatFile.get_text()),
            _('Set History Viewer history.dat File'), 
            self.txeHstDatFile.set_text)
            
    def on_btnHstLayout_clicked(self, widget, *args):
        """set mame history viewer layout file"""
        self.open_file_dialog(
            self.get_path(self.txeHstLayout.get_text()),
            _('Set History Viewer Layout File'), 
            self.txeHstLayout.set_text)
            
    def on_btnCPVIni_clicked(self, widget, *args):
        """set mame control panel viewer controls.ini file"""
        self.open_file_dialog(
            self.get_path(self.txeCPVIni.get_text()),
            _('Set Control Panel Viewer controls.ini File'), 
            self.txeCPVIni.set_text)
            
    def on_btnCPVLayout_clicked(self, widget, *args):
        """set mame control panel viewer layout file"""
        self.open_file_dialog(
            self.get_path(self.txeCPVLayout.get_text()),
            _('Set Control Panel Viewer Layout File'), 
            self.txeCPVLayout.set_text)
            
    def load_settings(self, default_emu='mame'):
        """load wahcade settings"""
        #build list of emulators
        self.emu_lists = []
        self.emu_game_lists = []
        self.current_emu = None
        self.current_emu_list = None
        emu_ini_files = glob.glob(os.path.join(self.config_dir, 'ini', '*[!0-9].ini'))
        for emu_ini in emu_ini_files:
            ini = MameWahIni(emu_ini)
            self.emu_lists.append(
                [ini.get('emulator_title'),
                os.path.splitext(os.path.basename(emu_ini))[0],
                ini])
        self.emu_lists.sort()
        #load emu combo
        l = ['%s (%s.ini)' % (e[0], e[1]) for e in self.emu_lists]
        self.setup_combo_box(self.cboEmu, l)
        #wahcade
        self.txeWCLayoutDir.set_text(self.wahcade_ini.get('layout'))
        self.spnWCScrDelay.set_value(self.wahcade_ini.getint('delay'))
        self.spnWCScrSlide.set_value(self.wahcade_ini.getint('slide_duration'))
        self.spnWCMovieDelay.set_value(self.wahcade_ini.getint('delay_before_movie_preview'))
        self.chkWCMovieAudio.set_active((self.wahcade_ini.getint('movie_volume') > 0))
        self.chkWCMouseCursor.set_active((self.wahcade_ini.getint('show_cursor') == 1))
        self.chkWCWrapLists.set_active((self.wahcade_ini.getint('wrap_list') == 1))
        self.chkWCScaleImages.set_active((self.wahcade_ini.getint('keep_image_aspect') == 1))
        #set emu
        set_idx = [idx for idx, e in enumerate(self.emu_lists) if e[1] == default_emu][0]
        self.cboEmu.set_active(set_idx)
        #mame history viewer
        self.txeHstDatFile.set_text(self.histview_ini.get('history_dat_file'))
        self.txeHstLayout.set_text(self.histview_ini.get('history_layout'))
        #mame cp viewer
        self.txeCPVIni.set_text(self.cpviewer_ini.get('controls_ini_file'))
        self.txeCPVLayout.set_text(self.cpviewer_ini.get('viewer_layout'))
        #load keys
        self.chkKeysUseKeyboard.set_active((self.ctrlr_ini.getint('keyboard') == 1))
        self.chkKeysUseMouse.set_active((self.ctrlr_ini.getint('mouse') == 1))
        self.chkKeysUseJoystick.set_active((self.ctrlr_ini.getint('joystick') == 1))
        self.populate_keys()
        
    def load_emulator_settings(self, ini_name, emu_ini, default_list=0):
        """load emu settings"""
        self.txeEmuTitle.set_text(emu_ini.get('emulator_title'))
        self.txeEmuExe.set_text(emu_ini.get('emulator_executable'))
        self.txeEmuCmdLine.set_text(emu_ini.get('commandline_format'))
        self.txeEmuRomExt.set_text(emu_ini.get('rom_extension'))
        self.txeEmuRomDir.set_text(emu_ini.get('rom_path'))
        #list gen type
        ini_lgen = emu_ini.get('list_generation_method')
        lgen_idx = [idx for idx, r in enumerate(self.emu_list_gen_types) if ini_lgen in r[0]][0]
        self.cboEmuListGen.set_active(lgen_idx)
        #artwork
        for idx, emu_art in enumerate(self.emu_artwork_txe):
            emu_art.set_text(emu_ini.get('artwork_%s_image_path' % (idx + 1)))
        self.txeEmuMovieDir.set_text(emu_ini.get('movie_preview_path'))
        self.spnEmuMovieNum.set_value(emu_ini.getint('movie_artwork_no'))
        self.cboeEmuExtApp1.child.set_text(emu_ini.get('app_1_executable'))
        self.txeEmuExtApp1.set_text(emu_ini.get('app_1_commandline_format'))
        self.cboeEmuExtApp2.child.set_text(emu_ini.get('app_2_executable'))
        self.txeEmuExtApp2.set_text(emu_ini.get('app_2_commandline_format'))
        self.cboeEmuExtApp3.child.set_text(emu_ini.get('app_3_executable'))
        self.txeEmuExtApp3.set_text(emu_ini.get('app_3_commandline_format'))
        self.txeEmuExtAuto.set_text(emu_ini.get('auto_launch_apps'))
        #screen saver
        ini_scr = emu_ini.get('saver_type')
        scr_idx = [idx for idx, r in enumerate(self.emu_scrsave_types) if r[0] == ini_scr][0]
        self.cboEmuScrSaver.set_active(scr_idx)
        self.txeEmuScrMovieDir.set_text(emu_ini.get('movie_path'))
        #load lists
        i = 0
        self.emu_game_lists = []
        while True:
            ini_file = os.path.join(self.config_dir, 'ini', '%s-%s.ini' % (ini_name, i))
            if os.path.exists(ini_file):
                list_ini = MameWahIni(ini_file)
                self.emu_game_lists.append([list_ini.get('list_title'), list_ini])
                i += 1
            else:
                break
        l = ['%s: %s' % (i, r[0]) for i, r in enumerate(self.emu_game_lists)]
        self.setup_combo_box(self.cboEmuLists, l)
        self.cboEmuLists.set_active(default_list)
        #mame only
        if ini_name == 'mame':
            self.txeMameXMLFile.set_text(emu_ini.get('dat_file'))
            self.txeMameCatver.set_text(emu_ini.get('catver_ini_file'))
            
    def load_emulator_list_settings(self, emu_list_idx, emu_list_ini):
        """load emulator list"""
        self.txeEmuListTitle.set_text(emu_list_ini.get('list_title'))
        #default or user-def list
        if emu_list_idx >= 1:
            #user defined list
            list_type = emu_list_ini.get('list_type')
            type_idx = [i for i, r in enumerate(self.emu_list_types) if r[0] == list_type][0]
            self.cboEmuListType.set_sensitive(True)
        else:
            #default list
            type_idx = -1
            self.cboEmuListType.set_sensitive(False)
        #set list type
        self.cboEmuListType.set_active(type_idx)
        self.chkEmuListCycle.set_active(emu_list_ini.getint('cycle_list') == 1)
        
    def save_wahcade_settings(self):
        """save wahcade settings"""
        #shift focus in order for spinner controls to update
        self.txeWCLayoutDir.grab_focus()
        #wahcade
        self.wahcade_ini.set('layout', self.txeWCLayoutDir.get_text())
        self.wahcade_ini.set('delay', int(self.spnWCScrDelay.get_value()))
        self.wahcade_ini.set('slide_duration', int(self.spnWCScrSlide.get_value()))
        self.wahcade_ini.set('delay_before_movie_preview', int(self.spnWCMovieDelay.get_value()))
        if self.chkWCMovieAudio.get_active():
            self.wahcade_ini.set('movie_volume', 100)
        else:
            self.wahcade_ini.set('movie_volume', 0)
        self.wahcade_ini.set('show_cursor', int(self.chkWCMouseCursor.get_active()))
        self.wahcade_ini.set('wrap_list', int(self.chkWCWrapLists.get_active()))
        self.wahcade_ini.set('keep_image_aspect', int(self.chkWCScaleImages.get_active()))
        #save ini
        self.wahcade_ini.write()
        #save controller settings
        #self.ctrlr_ini.set('keyboard', int(self.chkKeysUseKeyboard.get_active()))
        self.ctrlr_ini.set('mouse', int(self.chkKeysUseMouse.get_active()))
        #self.ctrlr_ini.set('joystick', int(self.chkKeysUseJoystick.get_active()))
        self.ctrlr_ini.write()
        
    def save_emulator_settings(self):
        """save current emulator settings"""
        ini_name = self.current_emu[1]
        emu_ini = self.current_emu[2]
        emu_ini.set('emulator_title', self.txeEmuTitle.get_text())
        emu_ini.set('emulator_executable', self.txeEmuExe.get_text())
        emu_ini.set('commandline_format', self.txeEmuCmdLine.get_text())
        if self.txeEmuRomExt.get_text().startswith('.'):
            self.txeEmuRomExt.set_text(self.txeEmuRomExt.get_text()[1:])
        emu_ini.set('rom_extension', self.txeEmuRomExt.get_text())
        emu_ini.set('rom_path', self.txeEmuRomDir.get_text())
        emu_ini.set('list_generation_method', self.emu_list_gen_types[self.cboEmuListGen.get_active()][0][0])
        for idx, emu_art in enumerate(self.emu_artwork_txe):
            emu_ini.set('artwork_%s_image_path' % (idx + 1), emu_art.get_text())
        emu_ini.set('movie_preview_path', self.txeEmuMovieDir.get_text())
        emu_ini.set('movie_artwork_no', int(self.spnEmuMovieNum.get_value()))
        emu_ini.set('app_1_executable', self.cboeEmuExtApp1.child.get_text())
        emu_ini.set('app_1_commandline_format', self.txeEmuExtApp1.get_text())
        emu_ini.set('app_2_executable', self.cboeEmuExtApp2.child.get_text())
        emu_ini.set('app_2_commandline_format', self.txeEmuExtApp2.get_text())
        emu_ini.set('app_3_executable', self.cboeEmuExtApp3.child.get_text())
        emu_ini.set('app_3_commandline_format', self.txeEmuExtApp3.get_text())
        emu_ini.set('auto_launch_apps', self.txeEmuExtAuto.get_text())
        emu_ini.set('saver_type', self.emu_scrsave_types[self.cboEmuScrSaver.get_active()][0])
        emu_ini.set('movie_path', self.txeEmuScrMovieDir.get_text())
        #mame only
        if ini_name == 'mame':
            emu_ini.set('dat_file', self.txeMameXMLFile.get_text())
            emu_ini.set('catver_ini_file', self.txeMameCatver.get_text())
            #mame history viewer
            self.histview_ini.set('history_dat_file', self.txeHstDatFile.get_text())
            self.histview_ini.set('history_layout', self.txeHstLayout.get_text())
            #mame cp viewer
            self.cpviewer_ini.set('controls_ini_file', self.txeCPVIni.get_text())
            self.cpviewer_ini.set('viewer_layout', self.txeCPVLayout.get_text())
            #save
            self.histview_ini.write()
            self.cpviewer_ini.write()
        #save emu in ifiles
        emu_ini.write()
        
    def save_emulator_list_settings(self):
        """save emulator game list"""
        emu_list_ini = self.current_emu_list[1] 
        emu_list_ini.set('list_title', self.txeEmuListTitle.get_text())
        if self.emu_game_lists.index(self.current_emu_list) >= 1:
            list_type = self.emu_list_types[self.cboEmuListType.get_active()][0]
            emu_list_ini.set('list_type', list_type)
        emu_list_ini.set('cycle_list', int(self.chkEmuListCycle.get_active()))
        #save
        emu_list_ini.write()
        
    def create_new_emulator_files(self, emulator_filename):
        """create a new set of emulator files"""
        emu_ini_name = os.path.splitext(os.path.basename(emulator_filename))[0]
        #copy template files to ini dir
        shutil.copy2(
            os.path.join(APP_PATH, 'templates', 'new_emulator.ini'), 
            os.path.join(self.config_dir, 'ini', '%s.ini' % emu_ini_name))
        shutil.copy2(
            os.path.join(APP_PATH, 'templates', 'new_emulator-0.ini'), 
            os.path.join(self.config_dir, 'ini', '%s-0.ini' % emu_ini_name))
        #save & then reload
        self.on_mnuFSave_activate()
        self.load_settings(default_emu = emu_ini_name)
            
    def populate_keys(self):
        """pop keys list"""
        self.lsKeys.clear()
        #for each 
        for ck, cv in self.ctrlr_ini.ini_dict.items():
            lv = cv.strip(' "').split(' | ')
            kb_vals = [v for v in lv if v.startswith('DIK_')]
            mouse_vals = [v for v in lv if v.startswith('MOUSE_')]
            key_desc = ''
            #get key descriptions
            for keyval in kb_vals:
                keyname = [k for k, v in mamewah_keys.items() if keyval in v]
                if keyname != []:
                    if key_desc != '':
                        key_desc = '%s or %s' % (key_desc, keyname[0].upper())
                    else:
                        key_desc = '%s' % (keyname[0].upper())
            #get mouse descriptions
            for mouseval in mouse_vals:
                mousename = [k for k, v in _mouse_ctrls.items() if mouseval in v]
                if mousename != []:
                    if key_desc != '':
                        key_desc = '%s or %s' % (key_desc, mousename[0].upper())
                    else:
                        key_desc = '%s' % (mousename[0].upper())
            if ck.isupper():
                self.lsKeys.append((ck, key_desc))
                
    def open_file_dialog(self, default_filename, dlg_title, load_function):
        """open file dialog"""
        dlg = gtk.FileChooserDialog(
            title = dlg_title,
            action = gtk.FILE_CHOOSER_ACTION_OPEN,
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        ftr = gtk.FileFilter()
        ftr.set_name("All files (*.*)")
        ftr.add_pattern("*")
        dlg.add_filter(ftr)
        dlg.set_filter(ftr)
        ftr = gtk.FileFilter()
        ftr.set_name("Config files (*.ini)")
        ftr.add_pattern("*.ini")
        dlg.add_filter(ftr)
        ftr = gtk.FileFilter()
        ftr.set_name("XML files (*.xml)")
        ftr.add_pattern("*.xml")
        dlg.add_filter(ftr)
        ftr = gtk.FileFilter()
        ftr.set_name("Dat files (*.dat)")
        ftr.add_pattern("*.dat")
        dlg.add_filter(ftr)
        if os.path.exists(default_filename):
            dlg.set_filename(default_filename)
        else:
            dlg.set_current_folder(self.config_dir)
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            load_function(dlg.get_filename())
        dlg.destroy()

    def open_dir_dialog(self, default_dir, dlg_title, dlg_cb):
        """choose directory dialog"""
        dlg = gtk.FileChooserDialog(
            title = dlg_title,
            action = gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        if os.path.exists(default_dir):
            dlg.set_filename(default_dir)
        else:
            dlg.set_current_folder(self.config_dir)
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            dlg_cb(dlg.get_filename())
        dlg.destroy()

    def save_layout_dialog(self, default_filename, dlg_title, save_function):
        """save file as... dialog"""
        dlg = gtk.FileChooserDialog(
            title = dlg_title,
            action = gtk.FILE_CHOOSER_ACTION_SAVE,
            buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK))
        ftr = gtk.FileFilter()
        ftr.set_name("All files")
        ftr.add_pattern("*")
        dlg.add_filter(ftr)
        ftr = gtk.FileFilter()
        ftr.set_name("Ini files")
        ftr.add_pattern("*.ini")
        dlg.add_filter(ftr)
        dlg.set_filter(ftr)
        if gtk.check_version(2,8,0) is None:
            dlg.set_do_overwrite_confirmation(True)
        dlg.set_filename(default_filename)
        response = dlg.run()
        if response == gtk.RESPONSE_OK:
            save_function(dlg.get_filename())
        dlg.destroy()
        
    def iif(self, condition, if_true, if_false):
        if condition:
            return if_true
        else:
            return if_false
    
